/*
	Modified transaction structures for use by HP load testing and transaction file input/output.
	This file:
	- Is ANSI C compliant for use in HP LoadRunner VUGen
	- Uses display char arrays for numeric and date/timefields, so that values can be used in 
	  VUGen scripts (TE_type(char *) function) and in COBOL programs without major handling.

	This file is based on EGen TxnHarnessStructs.h but with:
	- Only transaction inputs
	- //const int converted to #define F_(to be compatible with vugen)
	- No other #includes
	- Char arrays only
*/

#ifndef HPZREFXPTXNSTRUCTS_H
#define HPZREFXPTXNSTRUCTS_H

// From "MiscConsts.h"
#define F_TradeUpdateMaxRows                      20   // Max number of rows for the frames
#define F_TradeUpdateFrame1MaxRows                F_TradeUpdateMaxRows

#define F_TradeLookupMaxRows                      20   // Max number of rows for the frames
#define F_TradeLookupFrame1MaxRows                F_TradeLookupMaxRows

// From "TableConsts.h"
//CUSTOMER table
#define F_cL_NAME_len       25
#define F_cF_NAME_len       20
#define F_cM_NAME_len       1

#define F_cTAX_ID_len 20

//BROKER table
#define F_cB_NAME_len F_cF_NAME_len + F_cM_NAME_len + F_cL_NAME_len + 3    // two spaces and one period

//COMPANY table
#define F_cCO_NAME_len 60

//INDUSTRY table
#define F_cIN_NAME_len 50

//SECURITY table
//const int cS_NAME_len = 70;
#define F_cSYMBOL_len 7 + 1 + 7  // base + separator + extended
#define F_cS_ISSUE_len 6

//SETTLEMENT table
//const int cSE_CASH_TYPE_len = 40;

//SECTOR table
#define F_cSC_NAME_len 30
//const int cSC_ID_len = 2;

//STATUS_TYPE table
#define F_cST_ID_len 4
//const int cST_NAME_len = 10;

//TAX RATE table
#define F_cTX_ID_len 4

//TRADE TYPE table
#define F_cTT_ID_len 3


// From TxnHarnessStructs.h
// Market-Feed
#define F_max_feed_len 20

// Broker-Volume
#define F_max_broker_list_len 40

// Data-Maintenance
#define F_max_table_name 30


// lengths to convert numerics to display fields - not including null terminator.
#define TIdent_charlen 19    // PIC S9(18) DISPLAY
#define TIdent_charfmt "% 019lld"

#define TTrade_charlen 19    // PIC S9(18) DISPLAY
#define TTrade_charfmt "% 019lld"

#define double_charlen 20    // PIC S9(16)V99 DISPLAY
#define double_charfmt "% 020.2f"

#define dateonly_charlen 10  // MM/DD/YYYY from TIMESTAMP_STRUCT, using CDateTime
#define dateonly_fmtcode 30  // produces the above string
#define datetime_charlen 23  // yyyy/mm/dd HH:MM:SS.FFF  from TIMESTAMP_STRUCT, using CDateTime
#define datetime_fmtcode 12  // produces the above string, which works in db2 and sql server
							 // mdy would fail with DB2 timestamp 

#define INT32_charlen 10     // S9(9) DISPLAY
#define INT32_charfmt "% 010i"

#define bool_charlen 1       // 9 DISPLAY
#define bool_charfmt "%01i"


/*
*   Broker-Volume
*/
typedef struct TXPBrokerVolumeTxnInput
{
    // Transaction level inputs
    char            broker_list[F_max_broker_list_len][F_cB_NAME_len+1];
    char            sector_name[F_cSC_NAME_len+1];
} TXPBrokerVolumeTxnInput;   

/*
*   Customer-Position
*/
typedef struct TXPCustomerPositionTxnInput
{
    char        acct_id_idx[TIdent_charlen+1];
    char        cust_id[TIdent_charlen+1];
    char		get_history[bool_charlen+1];	
    char        tax_id[F_cTAX_ID_len+1];
} TXPCustomerPositionTxnInput; 

/*
*   Data-Maintenance
*/
typedef struct TXPDataMaintenanceTxnInput
{
    char    acct_id[TIdent_charlen+1];
    char    c_id[TIdent_charlen+1];
    char    co_id[TIdent_charlen+1];
    char    day_of_month[INT32_charlen+1];
    char    vol_incr[INT32_charlen+1];
    char    symbol[F_cSYMBOL_len+1];
    char    table_name[F_max_table_name+1];
    char    tx_id[F_cTX_ID_len+1];
} TXPDataMaintenanceTxnInput;

/*
*   Market-Feed
*/
// MEE populates this structure
typedef struct TXPStatusAndTradeType
{
    char    status_submitted[F_cST_ID_len+1];
    char    type_limit_buy[F_cTT_ID_len+1];
    char    type_limit_sell[F_cTT_ID_len+1];
    char    type_stop_loss[F_cTT_ID_len+1];
} TXPStatusAndTradeType;


//Incoming order from SendToMarket interface.
//Both TradeOrder and MarketFeed send a request.
typedef struct TXPTradeRequest
{
    char                price_quote[double_charlen+1];
    char                trade_id[TTrade_charlen+1];
    char                trade_qty[INT32_charlen+1];
    char                eAction[1+1];	// enum is 1 digit
    char                symbol[F_cSYMBOL_len+1];
    char                trade_type_id[F_cTT_ID_len+1];

} TXPTradeRequest;


//A single entry on the ticker tape feed.
typedef struct TXPTickerEntry
{
    char                price_quote[double_charlen+1];
    char                trade_qty[INT32_charlen+1];
    char                symbol[F_cSYMBOL_len+1];
} TXPTickerEntry;

//Market-Feed data sent from MEE to sponsor provided SUT interface
typedef struct TXPMarketFeedTxnInput
{
    char                unique_symbols[INT32_charlen+1];
    TXPStatusAndTradeType StatusAndTradeType;
    TXPTickerEntry        Entries[F_max_feed_len];
} TXPMarketFeedTxnInput;

/*
*   Market-Watch
*/
typedef struct TXPMarketWatchTxnInput
{
    char              acct_id[TIdent_charlen+1];
    char              c_id[TIdent_charlen+1];
    char              ending_co_id[TIdent_charlen+1];
    char              starting_co_id[TIdent_charlen+1];
    char              start_day[dateonly_charlen+1];
    char              industry_name[F_cIN_NAME_len+1];
} TXPMarketWatchTxnInput;

/*
*   Security-Detail
*/
typedef struct TXPSecurityDetailTxnInput
{
    char                max_rows_to_return[INT32_charlen+1];
    char                access_lob_flag[bool_charlen+1];
    char                start_day[dateonly_charlen+1];
    char                symbol[F_cSYMBOL_len+1];
} TXPSecurityDetailTxnInput;  

/*
*   Trade-Lookup
*/
typedef struct TXPTradeLookupTxnInput
{
    char                trade_id[F_TradeLookupFrame1MaxRows][TTrade_charlen+1];
    char                acct_id[TIdent_charlen+1];
    char                max_acct_id[TIdent_charlen+1];
    char                frame_to_execute[INT32_charlen+1];           // which of the frames to execute
    char                max_trades[INT32_charlen+1];
    char                end_trade_dts[datetime_charlen+1];
    char                start_trade_dts[datetime_charlen+1];
    char                symbol[F_cSYMBOL_len+1];
} TXPTradeLookupTxnInput;

/*
*   Trade-Order
*/
typedef struct TXPTradeOrderTxnInput
{
    char            requested_price[double_charlen+1];
    char            acct_id[TIdent_charlen+1];
    char            is_lifo[INT32_charlen+1];
    char            roll_it_back[INT32_charlen+1];
    char            trade_qty[INT32_charlen+1];
    char            type_is_margin[INT32_charlen+1];
    char            co_name[F_cCO_NAME_len+1];
    char            exec_f_name[F_cF_NAME_len+1];
    char            exec_l_name[F_cL_NAME_len+1];
    char            exec_tax_id[F_cTAX_ID_len+1];
    char            issue[F_cS_ISSUE_len+1];
    char            st_pending_id[F_cST_ID_len+1];
    char            st_submitted_id[F_cST_ID_len+1];
    char            symbol[F_cSYMBOL_len+1];
    char            trade_type_id[F_cTT_ID_len+1];
} TXPTradeOrderTxnInput;

/*
*   Trade-Result
*/
//Trade-Result data sent from MEE to sponsor provided SUT interface
typedef struct TXPTradeResultTxnInput
{
    char        trade_price[double_charlen+1];
    char        trade_id[TTrade_charlen+1];
} TXPTradeResultTxnInput;

/*
*   Trade-Status
*/
typedef struct TXPTradeStatusTxnInput
{
    char                acct_id[TIdent_charlen+1];
} TXPTradeStatusTxnInput;

/*
*   Trade-Update
*/
typedef struct TXPTradeUpdateTxnInput
{
    char                trade_id[F_TradeUpdateFrame1MaxRows][TTrade_charlen+1];
    char                acct_id[TIdent_charlen+1];
    char                max_acct_id[TIdent_charlen+1];
    char                frame_to_execute[INT32_charlen+1];                   // which of the frames to execute
    char                max_trades[INT32_charlen+1];
    char                max_updates[INT32_charlen+1];
    char                end_trade_dts[datetime_charlen+1];
    char                start_trade_dts[datetime_charlen+1];
    char                symbol[F_cSYMBOL_len+1];
} TXPTradeUpdateTxnInput;

/*
*   Trade-Cleanup
*/
typedef struct TXPTradeCleanupTxnInput
{
    char                start_trade_id[TTrade_charlen+1];
    char                st_canceled_id[F_cST_ID_len+1];
    char                st_pending_id[F_cST_ID_len+1];
    char                st_submitted_id[F_cST_ID_len+1];
} TXPTradeCleanupTxnInput;


// Record definition
//	TODO: move to separate file

typedef struct TFileCommon
{
    char txnID[2+1];
    char recNum[9+1]; // record # up to 999999999
    char timeStamp[17+1]; // yyyymmddhhmmssfff (24 hour format)
} TFileCommon;

//	Trade Request message received by HPMEE from server programs
typedef struct TXPTradeRequestMsg
{
    char txnID[2+1];
	TXPTradeRequest TXPTradeRequest;
} TXPTradeRequestMsg;

typedef union
{
    TXPBrokerVolumeTxnInput     FileBrokerVolumeTxnInput;
    TXPCustomerPositionTxnInput FileCustomerPositionTxnInput;
    TXPMarketWatchTxnInput      FileMarketWatchTxnInput;
    TXPSecurityDetailTxnInput   FileSecurityDetailTxnInput;
    TXPTradeLookupTxnInput      FileTradeLookupTxnInput;
    TXPTradeOrderTxnInput       FileTradeOrderTxnInput;
    TXPTradeStatusTxnInput      FileTradeStatusTxnInput;
    TXPTradeUpdateTxnInput      FileTradeUpdateTxnInput;
} TXPCETxnInput;

typedef union
{
    TXPMarketFeedTxnInput       FileMarketFeedTxnInput;
    TXPTradeResultTxnInput      FileTradeResultTxnInput;
} TXPMEETxnInput;

typedef struct TFileCERecord
{
    TFileCommon Common;
	TXPCETxnInput TxnInput;
	char nullTerminator;
} TFileCERecord;

//	CE Txn returned to CE load generator
typedef struct TXPCETxn
{
    char txnID[2+1];
	TXPCETxnInput TxnInput;

} TXPCETxn, *pTXPCETxn;

typedef struct TXPMEETxn
{
    char txnID[2+1];
	TXPMEETxnInput TxnInput;

} TXPMEETxn, *pTXPMEETxn;


//	Structure of all accumulated transactions returned per call to CMEE::SubmitTradeRequest()
//  Assume a high upper limit of accumulated transactions.
//	This is used instead of a straight array of TXPMEETxn because the MF transaction is so much
//	larger than the TR, and the TR is much more prevalent.
#define MEETXNSET_TRMAX 250
#define MEETXNSET_MFMAX 25
typedef struct TXPMEETxnSet
{
	long trCount;
	long mfCount;
	//	These two arrays are links into the actual transactions below
	//  to preserve order of transactions to be run.
	char txnID[MEETXNSET_TRMAX+MEETXNSET_MFMAX] [2+1];
	long txnSubscript[MEETXNSET_TRMAX+MEETXNSET_MFMAX];

	TXPTradeResultTxnInput MEETradeResultTxnInput[MEETXNSET_TRMAX];
	TXPMarketFeedTxnInput  MEEMarketFeedTxnInput[MEETXNSET_MFMAX];
} TXPMEETxnSet, *pTXPMEETxnSet;

#endif  // #ifndef HPZREFXPTXNSTRUCTS_H
